@@ -39,23 +39,23 @@ test_that("Check create_workflow works properly", {
 
 })
 
-###### split_data
-
-test_that("Check split_data works properly", {
-
-  model_object_bin$modify("tuner", "Bayesian Optimization")
-  model_object_reg$modify("tuner", "Grid Search CV")
-
-  split_data_bin <- split_data(model_object_bin)
-  split_data_reg <- split_data(model_object_reg)
-
-  expect_equal(class(split_data_bin$sampling_method$splits[[1]]), c("vfold_split", "rsplit"))
-  expect_equal(split_data_bin$final_split, rbind(model_object_bin$train_data, model_object_bin$validation_data))
-
-  expect_equal(class(split_data_reg$sampling_method$splits[[1]]), c("vfold_split", "rsplit") )
-  expect_equal(split_data_reg$final_split, model_object_reg$train_data)
-
-})
+# ###### split_data
+# 
+# test_that("Check split_data works properly", {
+# 
+#   model_object_bin$modify("tuner", "Bayesian Optimization")
+#   model_object_reg$modify("tuner", "Grid Search CV")
+# 
+#   split_data_bin <- split_data(model_object_bin)
+#   split_data_reg <- split_data(model_object_reg)
+# 
+#   expect_equal(class(split_data_bin$sampling_method$splits[[1]]), c("vfold_split", "rsplit"))
+#   expect_equal(split_data_bin$final_split, rbind(model_object_bin$train_data, model_object_bin$validation_data))
+# 
+#   expect_equal(class(split_data_reg$sampling_method$splits[[1]]), c("vfold_split", "rsplit") )
+#   expect_equal(split_data_reg$final_split, model_object_reg$train_data)
+# 
+# })
 
 
 ##### create_metric_set
